/***
 * Excerpted from "iCloud for Developers",
 * published by The Pragmatic Bookshelf.
 * Copyrights apply to this code. It may not be used to create training material, 
 * courses, books, articles, and the like. Contact us if you are in doubt.
 * We make no guarantees that this code is fit for any purpose. 
 * Visit http://www.pragmaticprogrammer.com/titles/cricloud for more book information.
***/
//
//  SMMasterViewController.m
//  Grocery
//
//  Created by cesarerocchi on 6/6/12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//


#import "SMMasterViewController.h"
#import "SMDetailViewController.h"
#define kFILENAME @"grocery.gro"

@interface SMMasterViewController () {
    NSMutableArray *_objects;
    NSMetadataQuery *_query;
}

- (void) loadData:(NSMetadataQuery *)query;
@end
@implementation SMMasterViewController

- (void) loadGrocery {
    _query = [[NSMetadataQuery alloc] init];
        
    [_query setSearchScopes:[NSArray arrayWithObject:
                             NSMetadataQueryUbiquitousDocumentsScope]];
    NSPredicate *pred = [NSPredicate predicateWithFormat:@"%K == %@", 
                         NSMetadataItemFSNameKey, 
                         kFILENAME];
    [_query setPredicate:pred];
   
    [[NSNotificationCenter defaultCenter] 
        addObserver:self 
           selector:@selector(queryDidFinishGathering:) 
               name:NSMetadataQueryDidFinishGatheringNotification 
             object:_query];
    
    [_query startQuery];
}

- (void) queryDidFinishGathering:(NSNotification *) notification {
    NSMetadataQuery *query = [notification object];
    
    [query disableUpdates];
    [query stopQuery];
    
    [[NSNotificationCenter defaultCenter] 
     removeObserver:self 
     name:NSMetadataQueryDidFinishGatheringNotification 
     object:query];
    
    _query = nil;
    
    [self loadData:query];
}

- (void) loadData:(NSMetadataQuery *)query {
    SMGroceryItem *groceryItem;

    if ([query resultCount] == 1) {
        NSMetadataItem *queryItem = [query resultAtIndex:0];
        NSURL *itemUrl = [queryItem valueForAttribute:NSMetadataItemURLKey];
        groceryItem = [[SMGroceryItem alloc] initWithFileURL:itemUrl];
        
        [_objects addObject:groceryItem];
        
        [groceryItem openWithCompletionHandler:^(BOOL success) {
            if (success) {
                NSLog(@"item opened");
                [self.tableView reloadData];
            } else {
                NSLog(@"failed opening item");
            }
        }];
    }
       
	else {
        // Getting iCloud container
        NSURL *ubiqContainer = [[NSFileManager defaultManager] 
                                URLForUbiquityContainerIdentifier:nil];
        
        // Building the URL for the new item
        NSURL *newItemUrl = [[ubiqContainer 
                             URLByAppendingPathComponent:@"Documents"] 
                             URLByAppendingPathComponent:kFILENAME];
        
        // Creating the new item 
        groceryItem = [[SMGroceryItem alloc] initWithFileURL:newItemUrl];
        
        // Saving the new item
        [groceryItem saveToURL:[groceryItem fileURL] 
              forSaveOperation:UIDocumentSaveForCreating 
             completionHandler:^(BOOL success) {
                 if (success) {
                     NSLog(@"new item created");
                     
                     // Opening the new item
                     [groceryItem openWithCompletionHandler:^(BOOL success) {
                         NSLog(@"New item opened");
                         [self.tableView reloadData];
                     }];
                 } 
         }];
        
        [_objects addObject:groceryItem];
    }
}

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        self.title = @"Grocery List";
        if ([[UIDevice currentDevice] userInterfaceIdiom] == UIUserInterfaceIdiomPad) {
            self.clearsSelectionOnViewWillAppear = NO;
            self.contentSizeForViewInPopover = CGSizeMake(320.0, 600.0);
        }
    }
    return self;
}
						
- (void)viewDidLoad
{
    [super viewDidLoad];
    UIBarButtonItem *addButton =
    [[UIBarButtonItem alloc]
     initWithBarButtonSystemItem:UIBarButtonSystemItemAdd
     target:self
     action:@selector(insertNewObject:)];
    
    self.navigationItem.rightBarButtonItem = addButton;
	
    _objects = [[NSMutableArray alloc] init];
    [self loadGrocery];
}


- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    if ([[UIDevice currentDevice] userInterfaceIdiom] == UIUserInterfaceIdiomPhone) {
        return (interfaceOrientation != UIInterfaceOrientationPortraitUpsideDown);
    } else {
        return YES;
    }
}

- (void)insertNewObject:(id)sender
{

}

#pragma mark - Table View

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    return _objects.count;
}

// Customize the appearance of table view cells.
- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    static NSString *CellIdentifier = @"Cell";
    
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    if (cell == nil) {
        cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:CellIdentifier];
        if ([[UIDevice currentDevice] userInterfaceIdiom] == UIUserInterfaceIdiomPhone) {
            cell.accessoryType = UITableViewCellAccessoryDisclosureIndicator;
        }
    }


    SMGroceryItem *groceryItem = [_objects objectAtIndex:indexPath.row];
    NSLog(@"name is %@", groceryItem.itemName);
    cell.textLabel.text = groceryItem.itemName;
    return cell;
}

- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath
{
    // Return NO if you do not want the specified item to be editable.
    return YES;
}

- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath
{
    if (editingStyle == UITableViewCellEditingStyleDelete) {
        [_objects removeObjectAtIndex:indexPath.row];
        [tableView deleteRowsAtIndexPaths:[NSArray arrayWithObject:indexPath] withRowAnimation:UITableViewRowAnimationFade];
    } else if (editingStyle == UITableViewCellEditingStyleInsert) {
        // Create a new instance of the appropriate class, insert it into the array, and add a new row to the table view.
    }
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    NSDate *object = [_objects objectAtIndex:indexPath.row];
    if ([[UIDevice currentDevice] userInterfaceIdiom] == UIUserInterfaceIdiomPhone) {
	    if (!self.detailViewController) {
	        self.detailViewController = [[SMDetailViewController alloc] initWithNibName:@"SMDetailViewController_iPhone" bundle:nil];
	    }
	    self.detailViewController.detailItem = object;
        [self.navigationController pushViewController:self.detailViewController animated:YES];
    } else {
        self.detailViewController.detailItem = object;
    }
}

@end
