/***
 * Excerpted from "iCloud for Developers",
 * published by The Pragmatic Bookshelf.
 * Copyrights apply to this code. It may not be used to create training material, 
 * courses, books, articles, and the like. Contact us if you are in doubt.
 * We make no guarantees that this code is fit for any purpose. 
 * Visit http://www.pragmaticprogrammer.com/titles/cricloud for more book information.
***/
//
//  SMDetailViewController.m
//  Grocery
//
//  Created by cesarerocchi on 6/6/12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#import "SMDetailViewController.h"

@interface SMDetailViewController ()

@property (strong, nonatomic) UIPopoverController *masterPopoverController;
@property (strong, nonatomic) UIView *documentStateView;

- (void)configureView;

@end

@implementation SMDetailViewController


#pragma mark - Managing the detail item

- (void)setDetailItem:(id)newDetailItem
{
    if (_detailItem != newDetailItem) {
        _detailItem = newDetailItem;
        
        [self configureView];
    }

    if (self.masterPopoverController != nil) {
        [self.masterPopoverController dismissPopoverAnimated:YES];
    }        
}

- (void)configureView {
    
    self.itemNameField.text = self.detailItem.itemName;
    
    switch (self.detailItem.documentState) {
    
        case UIDocumentStateClosed: NSLog(@"Closed");
            break;
        
        case UIDocumentStateNormal:
            NSLog(@"Normal");
            self.documentStateView.backgroundColor = [UIColor greenColor];
            break;
        
        case UIDocumentStateInConflict:
            NSLog(@"Conflict");
            self.documentStateView.backgroundColor = [UIColor redColor];
            break;
        
        case UIDocumentStateEditingDisabled:
            self.documentStateView.backgroundColor = [UIColor yellowColor];
            NSLog(@"Editing disabled");
            break;
        
        case UIDocumentStateSavingError:
            NSLog(@"Saving error");
            break;
        
        default:
            break;
            
    }
    
    if (self.detailItem.documentState == UIDocumentStateInConflict) {
	
        NSFileVersion *currentFileVersion = [NSFileVersion
                                             currentVersionOfItemAtURL:
                                             [self.detailItem fileURL]];
        NSArray *otherVersions = [NSFileVersion
                                  otherVersionsOfItemAtURL:
                                  [self.detailItem fileURL]];
        SMGroceryItem *i = [[SMGroceryItem alloc]
                            initWithFileURL:
                            [currentFileVersion URL]];
        
        [i openWithCompletionHandler:^(BOOL success) {
            NSLog(@"current version content is %@", i.itemName);
        }];
        
        for (NSFileVersion *c in otherVersions) { 
            SMGroceryItem *o = [[SMGroceryItem alloc] initWithFileURL:[c URL]];
            [o openWithCompletionHandler:^(BOOL success) {   
                NSLog(@"other version content is %@", o.itemName); 
            }];
        }}
}

- (void)viewDidLoad {
    [super viewDidLoad];
    self.documentStateView = [[UIView alloc]
                              initWithFrame:CGRectMake(5, 30, 10, 10)];
    self.documentStateView.backgroundColor = [UIColor clearColor];
    
    [self.view addSubview:self.documentStateView];
    UIBarButtonItem *saveButton =
      [[UIBarButtonItem alloc]
         initWithBarButtonSystemItem:UIBarButtonSystemItemSave
                              target:self
                              action:@selector(saveItem:)];
    self.navigationItem.rightBarButtonItem = saveButton;
    
    if ([[UIDevice currentDevice] userInterfaceIdiom] ==
        UIUserInterfaceIdiomPhone) {
        self.itemNameField = [[UITextField alloc]
                              initWithFrame:CGRectMake(20, 20, 280, 31)]; 
    } else {
        self.itemNameField = [[UITextField alloc]
                              initWithFrame:CGRectMake(20, 20, 340, 31)]; 
    }
    self.itemNameField.borderStyle = UITextBorderStyleRoundedRect;
    self.itemNameField.clearButtonMode = UITextFieldViewModeWhileEditing;
    self.itemNameField.delegate = self;
    [self.view addSubview:self.itemNameField];
    [self configureView];
    
    [[NSNotificationCenter defaultCenter]
      addObserver:self
         selector:@selector(configureView)
             name:UIDocumentStateChangedNotification
           object:nil];
}

- (void)textFieldDidBeginEditing:(UITextField *)textField {

    [[NSNotificationCenter defaultCenter]
        removeObserver:self
                  name:UIDocumentStateChangedNotification
                object:nil];
    
    
}

- (void) itemHasChanged:(id) sender {

    if (self.detailItem) {
        self.detailItem = [sender object];
        [self configureView];
    }
    
    if (self.detailItem.documentState == UIDocumentStateInConflict) {   
	
        [[NSNotificationCenter defaultCenter]
          removeObserver:self
                    name:UIDocumentStateChangedNotification
                  object:nil];
    }
}


- (void) saveItem:(id) sender {

    // 1. Setting the winning version
    self.detailItem.itemName = self.itemNameField.text;
    
    if (self.detailItem.documentState == UIDocumentStateInConflict) {
        NSArray *conflicts = [NSFileVersion
                              unresolvedConflictVersionsOfItemAtURL:
                              [self.detailItem fileURL]];
        
        // 2. Marking conflicts as resolved
        for (NSFileVersion *conflict in conflicts) {
            conflict.resolved = YES; 
        }
        
        // 3. Removing other versions
        NSError *error = nil;
        BOOL ok = [NSFileVersion removeOtherVersionsOfItemAtURL:
                   [self.detailItem fileURL] error:&error];
        
        if (!ok) {
            NSLog(@"Can't remove other versions: %@", error); 
        }
    }
    
    [self.detailItem saveToURL:self.detailItem.fileURL
              forSaveOperation:UIDocumentSaveForOverwriting
             completionHandler:^(BOOL success) {
                 
                 if (success) {
                     NSLog(@"item saved ");
                     if ([[UIDevice currentDevice] userInterfaceIdiom] ==
                         UIUserInterfaceIdiomPhone) {
                         [self.navigationController
                          popViewControllerAnimated:YES];
                     } 
                 }
             }];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    if ([[UIDevice currentDevice] userInterfaceIdiom] == UIUserInterfaceIdiomPhone) {
        return (interfaceOrientation != UIInterfaceOrientationPortraitUpsideDown);
    } else {
        return YES;
    }
}

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        self.title = NSLocalizedString(@"Item Detail", @"Detail");
    }
    return self;
}
							
#pragma mark - Split view

- (void)splitViewController:(UISplitViewController *)splitController willHideViewController:(UIViewController *)viewController withBarButtonItem:(UIBarButtonItem *)barButtonItem forPopoverController:(UIPopoverController *)popoverController
{
    barButtonItem.title = @"Grocery list";
    [self.navigationItem setLeftBarButtonItem:barButtonItem animated:YES];
    self.masterPopoverController = popoverController;
}

- (void)splitViewController:(UISplitViewController *)splitController willShowViewController:(UIViewController *)viewController invalidatingBarButtonItem:(UIBarButtonItem *)barButtonItem
{
    // Called when the view is shown again in the split view, invalidating the button and popover controller.
    [self.navigationItem setLeftBarButtonItem:nil animated:YES];
    self.masterPopoverController = nil;
}

@end
