/***
 * Excerpted from "iCloud for Developers",
 * published by The Pragmatic Bookshelf.
 * Copyrights apply to this code. It may not be used to create training material, 
 * courses, books, articles, and the like. Contact us if you are in doubt.
 * We make no guarantees that this code is fit for any purpose. 
 * Visit http://www.pragmaticprogrammer.com/titles/cricloud for more book information.
***/
//
//  SMAppDelegate.m
//  Grocery
//
//  Created by Cesare Rocchi on 12/17/12.
//  Copyright (c) 2012 Cesare Rocchi. All rights reserved.
//

#import "SMAppDelegate.h"

#import "SMMasterViewController.h"

#import "SMDetailViewController.h"

@implementation SMAppDelegate

@synthesize managedObjectContext = _managedObjectContext;
@synthesize managedObjectModel = _managedObjectModel;
@synthesize persistentStoreCoordinator = _persistentStoreCoordinator;

- (BOOL)application:(UIApplication *)application didFinishLaunchingWithOptions:(NSDictionary *)launchOptions
{
    self.window = [[UIWindow alloc] initWithFrame:[[UIScreen mainScreen] bounds]];
    // Override point for customization after application launch.
    if ([[UIDevice currentDevice] userInterfaceIdiom] == UIUserInterfaceIdiomPhone) {
        SMMasterViewController *masterViewController = [[SMMasterViewController alloc] initWithNibName:@"SMMasterViewController_iPhone" bundle:nil];
        self.navigationController = [[UINavigationController alloc] initWithRootViewController:masterViewController];
        self.window.rootViewController = self.navigationController;
        masterViewController.managedObjectContext = self.managedObjectContext;
    } else {
        SMMasterViewController *masterViewController = [[SMMasterViewController alloc] initWithNibName:@"SMMasterViewController_iPad" bundle:nil];
        UINavigationController *masterNavigationController = [[UINavigationController alloc] initWithRootViewController:masterViewController];
        
        SMDetailViewController *detailViewController = [[SMDetailViewController alloc] initWithNibName:@"SMDetailViewController_iPad" bundle:nil];
        UINavigationController *detailNavigationController = [[UINavigationController alloc] initWithRootViewController:detailViewController];
    	
    	masterViewController.detailViewController = detailViewController;
        
        self.splitViewController = [[UISplitViewController alloc] init];
        self.splitViewController.delegate = detailViewController;
        self.splitViewController.viewControllers = @[masterNavigationController, detailNavigationController];
        
        self.window.rootViewController = self.splitViewController;
        masterViewController.managedObjectContext = self.managedObjectContext;
    }
    [self.window makeKeyAndVisible];
    return YES;
}

- (void)applicationWillResignActive:(UIApplication *)application
{
    // Sent when the application is about to move from active to inactive state. This can occur for certain types of temporary interruptions (such as an incoming phone call or SMS message) or when the user quits the application and it begins the transition to the background state.
    // Use this method to pause ongoing tasks, disable timers, and throttle down OpenGL ES frame rates. Games should use this method to pause the game.
}

- (void)applicationDidEnterBackground:(UIApplication *)application
{
    // Use this method to release shared resources, save user data, invalidate timers, and store enough application state information to restore your application to its current state in case it is terminated later. 
    // If your application supports background execution, this method is called instead of applicationWillTerminate: when the user quits.
}

- (void)applicationWillEnterForeground:(UIApplication *)application
{
    // Called as part of the transition from the background to the inactive state; here you can undo many of the changes made on entering the background.
}

- (void)applicationDidBecomeActive:(UIApplication *)application
{
    // Restart any tasks that were paused (or not yet started) while the application was inactive. If the application was previously in the background, optionally refresh the user interface.
}

- (void)applicationWillTerminate:(UIApplication *)application
{
    // Saves changes in the application's managed object context before the application terminates.
    [self saveContext];
}

- (void)saveContext
{
    NSError *error = nil;
    NSManagedObjectContext *managedObjectContext = self.managedObjectContext;
    if (managedObjectContext != nil) {
        if ([managedObjectContext hasChanges] && ![managedObjectContext save:&error]) {
             // Replace this implementation with code to handle the error appropriately.
             // abort() causes the application to generate a crash log and terminate. You should not use this function in a shipping application, although it may be useful during development. 
            NSLog(@"Unresolved error %@, %@", error, [error userInfo]);
            abort();
        } 
    }
}

#pragma mark - Core Data stack

- (NSManagedObjectContext *)managedObjectContext
{
    if (_managedObjectContext != nil) {
        return _managedObjectContext;
    }
    
    NSPersistentStoreCoordinator *coordinator =
    [self persistentStoreCoordinator];
    if (coordinator != nil) {
        // choose a concurrency type for the context
        NSManagedObjectContext* moc = [[NSManagedObjectContext alloc]
                                       initWithConcurrencyType:
                                       NSMainQueueConcurrencyType];
        
        [moc performBlockAndWait:^{
            // configure context properties
            [moc setPersistentStoreCoordinator: coordinator];
            [[NSNotificationCenter defaultCenter]
             
             addObserver:self
             selector:@selector(mergeChangesFromiCloud:)
             name:NSPersistentStoreDidImportUbiquitousContentChangesNotification
             object:coordinator];
        }];
        _managedObjectContext = moc;
    }
    return _managedObjectContext;
}

- (NSManagedObjectModel *)managedObjectModel
{
    if (_managedObjectModel != nil) {
        return _managedObjectModel;
    }
    NSURL *modelURL = [[NSBundle mainBundle] URLForResource:@"Grocery" withExtension:@"momd"];
    _managedObjectModel = [[NSManagedObjectModel alloc] initWithContentsOfURL:modelURL];
    return _managedObjectModel;
}

- (NSPersistentStoreCoordinator *)persistentStoreCoordinator
{
    if (_persistentStoreCoordinator != nil) {
        return _persistentStoreCoordinator;
    }
    
    _persistentStoreCoordinator = [[NSPersistentStoreCoordinator alloc]
                                   initWithManagedObjectModel:
                                   [self managedObjectModel]];
    
    NSString *storePath = [[self applicationDocumentsDirectory]
                           stringByAppendingPathComponent:@"Grocery.sqlite"];
    
    NSPersistentStoreCoordinator* psc = _persistentStoreCoordinator;
    
    dispatch_async(
       dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0),
       ^{
           
           NSURL *storeUrl = [NSURL fileURLWithPath:storePath];
           
           
           // building the path to store transaction logs
           NSFileManager *fileManager = [NSFileManager defaultManager];
           NSURL *transactionLogsURL = [fileManager
                                        URLForUbiquityContainerIdentifier:nil];
										
           NSString* coreDataCloudContent = [[transactionLogsURL path]
                                             stringByAppendingPathComponent:
                                             @"grocery_data"];
											 
           transactionLogsURL = [NSURL fileURLWithPath:coreDataCloudContent];
           
           //  Building the options array for the coordinator
           NSDictionary* options = @{NSPersistentStoreUbiquitousContentNameKey :
                                         @"com.studiomagnolia.coredata.grocery",
                                     NSPersistentStoreUbiquitousContentURLKey:
                                         transactionLogsURL,
                                     NSMigratePersistentStoresAutomaticallyOption:
                                         @YES
                                     };
           
           NSError *error = nil;
           
           [psc lock];
           
           if (![psc addPersistentStoreWithType:NSSQLiteStoreType
                                  configuration:nil
                                            URL:storeUrl
                                        options:options
                                          error:&error]) {
               
               NSLog(@"Core data error %@, %@", error, [error userInfo]);
               
           }
           
           [psc unlock];
           
           // post a notification to tell the main thread
           // to refresh the user interface
           dispatch_async(dispatch_get_main_queue(), ^{
               NSLog(@"persistent store added");
               [[NSNotificationCenter defaultCenter]
                postNotificationName:
                @"com.studiomagnolia.groceryItemsSynchronized"
                object:self
                userInfo:nil];
           });
       });
    return _persistentStoreCoordinator;
}

- (void)mergeChangesFromiCloud:(NSNotification *)notification { 
    NSManagedObjectContext* moc = [self managedObjectContext];
    NSDictionary *noteInfo = [notification userInfo];
    
    [moc performBlock:^{
        NSMutableDictionary *mergingPolicyResult = [NSMutableDictionary dictionary];
        [mergingPolicyResult setObject:noteInfo[NSInsertedObjectsKey]
                                forKey:NSInsertedObjectsKey];
        [mergingPolicyResult setObject:noteInfo[NSUpdatedObjectsKey]
                                forKey:NSUpdatedObjectsKey];
        [mergingPolicyResult setObject:[NSSet set] // Exclude deletions
                                forKey:NSDeletedObjectsKey];
        NSNotification *saveNotification =
        [NSNotification notificationWithName:notification.name
                                      object:self
                                    userInfo:mergingPolicyResult];
									
        [moc mergeChangesFromContextDidSaveNotification:saveNotification];
        [moc processPendingChanges]; 
    }];
}

#pragma mark - Application's Documents directory

- (NSString *)applicationDocumentsDirectory {
	return [NSSearchPathForDirectoriesInDomains(NSDocumentDirectory,
                                                NSUserDomainMask, YES) lastObject]; 
}

@end
